#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <windows.h>
#include <glew.h>
#include <il.h>
#include <GL/glui.h>
#include <io.h>
#include <fcntl.h> 

#include "common.h"
#include "FlowData.h"
#include "Transfer.h"
#include "FlowChannel.h"
#include "Streamlines.h"
using namespace std;

int main_window;
int g_iWidth = 900, g_iHeight = 600; // window dimensions
int controlsWidth = 160;
float xyRatio, winRatio;
int chX,chY,vel,pre,vor; //grid x, grid y, velocity, pessure, vorticity

FlowData* dataset;

void glutClose(int control);
void drawTransferFunction(void);
void drawPoints(void);
void drawStreamlines(std::vector<float*>);
void renderText(float x, float y, void *font, char *string);
int initTextures(std::string);

void theGame(void);
void drawBall(float x, float y, float size);
void drawRacket(float x, float y, float size);
void drawGates(float gap);
void drawBorders(void);

Transfer* tf;
int num;
int counter; 
int dividedBy = 200;
int draggingPoint = -1;
float* colorf;
float* grid;
bool active = false;

float shiftRacket=0;
float ballX=.0f, ballY=.0f, ballXSpeed=.0f, ballYSpeed=-0.02f;
float racketX=.0f, racketY=-.75f;
float gateGap=.0f;
float ballSize = .05f;
bool playGame=false;
bool lost = false, won=false;
int racketHit=0;
float streamlineInfluence = 10000.0f;

//Velocity Texture
const int sizeX = 128;
const int sizeY = 128;
float* xyz = new float[sizeX*sizeY*3];

Streamlines* sl;
int streamLineDist = 30;
float streamLineStep = 0.01;
int maxStreamLineSteps = 50;
int drawArrows = 0;
int drawEuler = 0;
int drawRK = 0;
int arrow_distance = 20;
int hurricane = 0;
int adaptPt = 0;
int arrow_size = 10;
bool streamLineEChanged = true;
bool streamLineRKChanged = true;
std::vector<float*> euler;
std::vector<float*> rungeKutta;

GLUI *glui;
GLUI_RadioGroup* group1;
GLUI_Scrollbar* slider;
GLuint vertex;
GLuint fragment;
GLuint p;
GLuint g_newTexture; // new texture
GLuint g_uExampleTexture; //Example texture.. maybe not needed?
GLuint g_PointTexture;
GLuint g_ArrowTexture;
GLuint g_BallTexture;
GLuint velTexture;
GLint doShade;
GLint useData;
GLint plotArrows;
GLint plotBall;
GLint arrowSize;
GLint arrowAngle;
GLint drawHurricane;
GLint adaptPtSize;

//-------------------------------------------------------------------------------------------//
//------------------------------------GLUT STUFF---------------------------------------------//
void glutIdle(void)
{
	if ( glutGetWindow() != main_window ) 
		glutSetWindow(main_window);

	//trigger the display function (glutDisplay is called in the next iteration of the glutMainLoop)
	glutPostRedisplay();
}


void glutDisplay(void)
{
	GLUI_Master.sync_live_all();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

	glUniform1iARB(doShade, 0);
	glUniform1iARB(plotArrows, 0);

	if (g_newTexture)
	{
		glActiveTexture(GL_TEXTURE1);
		glEnable(GL_TEXTURE_1D);
		glTexImage1D(GL_TEXTURE_1D, 0, GL_RGB, num, 0, GL_RGB, GL_FLOAT,(void*)colorf);
		glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glDisable(GL_TEXTURE_1D);
	}

	float diffx, diffy;
	glUniform1iARB(doShade, 1);
	if (g_uExampleTexture) //texture is created
	{
		glActiveTexture(GL_TEXTURE0);
		glEnable(GL_TEXTURE_2D);

		if (winRatio > xyRatio)
		{
			float diff = (float)dataset->getChannel(0)->geom->getDimX()/(float)(g_iWidth-controlsWidth);
			glBegin(GL_QUADS);
			glTexCoord3f(0.0f,1.0f, 0.0f);
			glVertex3f(-1.0f*diff,1.0f,-1.0f);
			glTexCoord3f(1.0f,1.0f, 0.0f);
			glVertex3f(1.0f*diff,1.0f,-1.0f);
			glTexCoord3f(1.0f,0.0f, 1.0f);
			glVertex3f(1.0f*diff,-1.0f,-1.0f);
			glTexCoord3f(0.0f,0.0f, 1.0f);
			glVertex3f(-1.0f*diff,-1.0f,-1.0f);
			glEnd();
		} else {
			float diff = (float)dataset->getChannel(0)->geom->getDimY()/(float)(g_iHeight-100);
			glBegin(GL_QUADS);
			glTexCoord3f(0.0f,1.0f, 0.0f);
			glVertex3f(-1.0f,1.0f*diff,-1.0f);
			glTexCoord3f(1.0f,1.0f, 0.0f);
			glVertex3f(1.0f,1.0f*diff,-1.0f);
			glTexCoord3f(1.0f,0.0f, 1.0f);
			glVertex3f(1.0f,-1.0f*diff,-1.0f);
			glTexCoord3f(0.0f,0.0f, 1.0f);
			glVertex3f(-1.0f,-1.0f*diff,-1.0f);
			glEnd();
		}
		glDisable(GL_TEXTURE_2D);
	}

	glUniform1iARB(doShade, 0);
	
	//STREAMLINES
	if (drawEuler || drawRK)
	{
		sl = new Streamlines(dataset, streamLineDist, streamLineStep, maxStreamLineSteps, g_iWidth-controlsWidth);
		if(drawEuler)
		{
			glColor4f(1.0f, .0f, .0f, 1.0f );
			if (streamLineEChanged) {
				euler = sl->getStreamLines(1);
				streamLineEChanged = false;
			}
			drawStreamlines(euler);
		}
		if (drawRK)
		{
			glColor4f(.0f, .0f, 1.0f, 1.0f );
			if (streamLineRKChanged) {
				rungeKutta = sl->getStreamLines(2);
				streamLineRKChanged = false;
			}
			drawStreamlines(rungeKutta);
		}
	}

	//ARROWS
	if(drawArrows)
	{
		glUniform1iARB(plotArrows, 1);
		glUniform1iARB(arrowSize, arrow_size);
		glUniform1iARB(adaptPtSize, adaptPt);

		glActiveTexture(GL_TEXTURE2);
		glEnable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

		if (g_ArrowTexture) //texture is created
		{
			glEnable(GL_POINT_SPRITE);
			glEnable(GL_VERTEX_PROGRAM_POINT_SIZE);
			glTexEnvf(GL_POINT_SPRITE_ARB,GL_COORD_REPLACE_ARB, GL_TRUE);
			float diffX = (float)dataset->getChannel(0)->geom->getDimX()/(float)(g_iWidth-controlsWidth);

			for(int i=1; i < dataset->getChannel(0)->geom->getDimX(); i++)
			{
				if(i%arrow_distance==0){
					for(int j=0; j < dataset->getChannel(0)->geom->getDimY(); j++)
					{
						if(j%arrow_distance==0)
						{
							float pointX = (((((float)i/dataset->getChannel(0)->geom->getDimX())*2)-1)*(diffX));//+(diffx/2);
							float pointY = ((((float)j/dataset->getChannel(0)->geom->getDimY())*2)-1);//+diffy;
							glColor3f(1.0f,0.0f,0.0f);
							int angle = j;
							glUniform1iARB(arrowAngle, angle);
							glBegin(GL_POINTS);
							glVertex3f(pointX,pointY,1.0f);
							glEnd();
						}
					}
				}
			}
			
			glDisable(GL_VERTEX_PROGRAM_POINT_SIZE);
			glDisable(GL_POINT_SPRITE);
		}
		glDisable(GL_BLEND);
		glDisable(GL_TEXTURE_2D);

		glUniform1iARB(plotArrows, 0);
	}

	glClearColor( .9f, .9f, .9f, 1.0f );
	drawTransferFunction();

	if(playGame)
		theGame();
	if(lost)
	{
		glColor3f(1.0f,.0f,1.0f);
		renderText(-.1f,.0f,GLUT_BITMAP_TIMES_ROMAN_24,"You lost!");
	}
	else if(won)
	{
		glColor3f(1.0f,.0f,1.0f);
		renderText(-.1f,.0f,GLUT_BITMAP_TIMES_ROMAN_24,"not bad!");
	}

	glutSwapBuffers();
}

void glutReshape(int x, int y)
{
	int tx, ty, tw, th;
	GLUI_Master.get_viewport_area( &tx, &ty, &tw, &th );
	glViewport( tx, ty, tw, th );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();

	g_iWidth = x;
	g_iHeight = y;

	xyRatio= (float)dataset->getChannel(chX)->geom->getDimX()/(float)dataset->getChannel(chX)->geom->getDimY();
	printf("xyRatio: %f\n",xyRatio);
	winRatio=(float)(g_iWidth-controlsWidth)/((float)g_iHeight-((float)g_iHeight/8));
	printf("winRatio: %f\n",winRatio);
}

void glutMouse(int button, int state, int x, int y)
{
	float width = (float)g_iWidth-controlsWidth;
	float height = (float)g_iHeight;
	float xValue = (x-(width/20))/((19*(width/20))-(width/20));
	xValue*=num;
	
	if(!active)
	{
		active!=active;
		if(x >= width/20 && x <= 19*(width/20))
		{
			if(y >= 177.5f*(height/200) && y <= 197.5f*(height/200))
			{
				bool onPoint=false;				
				//printf("mouse coords: %f:%f\n",xValue,y);
				for(unsigned int i=0; i<tf->points.size(); i++)
				{
					//printf("point coords: %f\n\n",tf->points.at(i)->getX());
					if(xValue > tf->points.at(i)->getX()-8.0f && xValue < tf->points.at(i)->getX()+8.0f)
					{
						//printf("until now, on point\n");
						if(y >= 177.5f*(height/200) && y <= 187.5f*(height/200))
						{
							//printf("draggingPoint!\n");
							draggingPoint = i;
							onPoint = true;
							if(button == 2)
							{
								Color *c = new Color(0.0f,0.0f,0.0f);
								Color::chooseColor(*c);	
								tf->points.at(draggingPoint)->color->SetNormalizedRed(c->GetNormalizedRed());
								tf->points.at(draggingPoint)->color->SetNormalizedGreen(c->GetNormalizedGreen());
								tf->points.at(draggingPoint)->color->SetNormalizedBlue(c->GetNormalizedBlue());
								//active=!active;
							}
						}
					}
				}
				if(!onPoint)
				{
					//active=!active;
					//printf("not on point!\n");
					Color *c = new Color(0.0f,0.0f,0.0f);
					if(Color::chooseColor(*c))
					{										
						ColorPoint* point = new ColorPoint(xValue,c);
						tf->addPoint(point);
					}					
				}				
			}	
		}
		tf->sortPoints();
		colorf = tf->colorInterpolation();
	}
	else
	{
		active=!active;
		tf->sortPoints();
		colorf = tf->colorInterpolation();
		draggingPoint = -1;
	}
}

void glutMotion (int x, int y)
{
	float width = (float)g_iWidth-controlsWidth;
	float height = (float)g_iHeight;
	float xValue = (x-(width/20))/((19*(width/20))-(width/20));
	xValue*=num;

	if(draggingPoint != 0 && draggingPoint != tf->points.size()-1 && draggingPoint != -1)
	{
		if(x < width/20 || x > 19*(width/20) || y < 177.5f*(height/200) || y > 197.5f*(height/200))
		{
			tf->removePoint(draggingPoint);
			draggingPoint=-1;
		}
		else
		{
			tf->points.at(draggingPoint)->setX(xValue);
		}
	}
	active=!active;
	colorf = tf->colorInterpolation();
}

void glutSpecialKey(int key, int x, int y)
{
	if (key == GLUT_KEY_LEFT)
    {
		shiftRacket -= 0.05f;
    }
	if (key == GLUT_KEY_RIGHT)
    {
		shiftRacket += 0.05f;
    }
}

void glutKeyboard(unsigned char key, int x, int y)
{
    //Close window on Esc
    if (key == unsigned char(27))
    {
        glutClose(1);
    }
}

void glutClose(int control)
{
    //memory clean up
    delete dataset;

	glDetachObjectARB(p,vertex); 
	glDetachObjectARB(p,fragment);
	glDeleteObjectARB(vertex);
	glDeleteObjectARB(fragment);
	glDeleteObjectARB(p); 

    exit(1);
}
//-----------------------------------GLUT STUFF----------------------------------------------//
//-------------------------------------------------------------------------------------------//

/**
* Draw transfer function on GUI.
* The transfer function is represented on the GUI by a color flow. The colors
* are selected by the user. The transfer function is then used for color coding
* of data.
*
*/
void drawTransferFunction()
{
	glUniform1iARB(doShade, 0);
	glDisable(GL_TEXTURE_1D);
	glDisable(GL_TEXTURE_2D);

	glBegin(GL_QUADS);
	glColor3f(.7f,.7f,.7f);
    glVertex3f(-1.0f,-.75f,-1.0f);
    glVertex3f(1.0f,-.75f,-1.0f);
    glVertex3f(1.0f,-1.0f,-1.0f);
    glVertex3f(-1.0f,-1.0f,-1.0f);
    glEnd();
   
    glBegin(GL_LINE_LOOP);
	glColor3f(.2f,.2f,.2f);
    glVertex3f(-.9f,-.775f,-1.0f);
    glVertex3f(.9f,-.775f,-1.0f);
    glVertex3f(.9f,-.875f,-1.0f);
    glVertex3f(-.9f,-.875f,-1.0f);
    glEnd();
    
	//LEGENDE//
	glColor3f(.0f,.0f,.0f);
	renderText(-.95f, -.95f, GLUT_BITMAP_HELVETICA_12, "wenig");
	renderText(-.55f, -.95f, GLUT_BITMAP_HELVETICA_12, "wenig/mittel");
	renderText(-.05f, -.95f, GLUT_BITMAP_HELVETICA_12, "mittel");
	renderText( .4f , -.95f, GLUT_BITMAP_HELVETICA_12, "mittel/viel");
	renderText( .85f, -.95f, GLUT_BITMAP_HELVETICA_12, "viel");

	  drawPoints();
}

/**
* Draw points on transfer function.
* When points are saved and loaded, they need to be drawn on the
* transfer function UI with this method.
*
*/
void drawPoints()
{
	for(unsigned int i = 0; i < (tf->points.size()-1); i++)
	{
		float x = (tf->points.at(i)->getX()/num);
		float xTwo = (tf->points.at(i+1)->getX()/num);
		glBegin(GL_POLYGON);
		glColor4f(tf->points.at(i)->getColor()->GetNormalizedRed(),tf->points.at(i)->getColor()->GetNormalizedGreen(),tf->points.at(i)->getColor()->GetNormalizedBlue(),0.0f);
		//glColor3f(1.0f,.0f,.0f);
		glVertex3f(((x*2.0f)-1.0f)*.9f,-.775f,-1.0f);
		glVertex3f(((x*2.0f)-1.0f)*.9f,-.874f,-1.0f);
		glColor4f(tf->points.at(i+1)->getColor()->GetNormalizedRed(),tf->points.at(i+1)->getColor()->GetNormalizedGreen(),tf->points.at(i+1)->getColor()->GetNormalizedBlue(),0.3f);
		//glColor3f(.0f,1.0f,.0f);
		glVertex3f(((xTwo*2.0f)-1.0f)*.9f,-.874f,-1.0f);
		glVertex3f(((xTwo*2.0f)-1.0f)*.9f,-.775f,-1.0f);
		glEnd();
	}

	glColor3f(0.0f,0.0f,0.0f);
	glBegin(GL_LINE_STRIP);
	for(unsigned int i=0; i<tf->points.size(); i++)
	{
		float x = (tf->points.at(i)->getX()/num);
		glVertex3f(((x*2.0f)-1.0f)*.9f,-.825f,-1.0f);
	}
	glEnd();

	for(unsigned int i=0; i<tf->points.size(); i++)
	{
		float x = (tf->points.at(i)->getX()/num);
		glColor3f(tf->points.at(i)->getColor()->GetNormalizedRed(), tf->points.at(i)->getColor()->GetNormalizedGreen(), tf->points.at(i)->getColor()->GetNormalizedBlue());
		glBegin(GL_QUADS);
		glVertex3f(((x*2.0f)-1.0f)*.9f-.02f,-.825f+.02,-1.0f);
		glVertex3f(((x*2.0f)-1.0f)*.9f+.02f,-.825f+.02,-1.0f);
		glVertex3f(((x*2.0f)-1.0f)*.9f+.02f,-.825f-.02,-1.0f);
		glVertex3f(((x*2.0f)-1.0f)*.9f-.02f,-.825f-.02,-1.0f);
		glEnd();

		glColor3f(1.0f,1.0f,1.0f);
		glBegin(GL_LINE_LOOP);
		glVertex3f(((x*2.0f)-1.0f)*.9f-.02f,-.825f+.02,-1.0f);
		glVertex3f(((x*2.0f)-1.0f)*.9f+.02f,-.825f+.02,-1.0f);
		glVertex3f(((x*2.0f)-1.0f)*.9f+.02f,-.825f-.02,-1.0f);
		glVertex3f(((x*2.0f)-1.0f)*.9f-.02f,-.825f-.02,-1.0f);
		glEnd();
	}
}

/**
* Render text on a glut window at a given position.
*
* @param float x
* @param float y
* @param void* font
* @param char* string
*/
void renderText(float x, float y, void *font, char *string)
{
	char *c;
	glRasterPos2f(x,y);
	for (c=string; *c != '\0'; c++)
	{
		glutBitmapCharacter(font, *c);
	}
}

//------------------------------------------------------------------------------//
//-----------------------------------THE GAME-----------------------------------//
/**
* The Game. This function contains all functionality of the game.
* The ball position is updated and the collision detection with the
* racket takes place here.
*
*/
void theGame()
{
	ballX += ballXSpeed;
	ballY += ballYSpeed;
	if(ballX >= racketX-0.2f+shiftRacket && ballX <= racketX+0.2f+shiftRacket && ballY <= racketY+0.1f)
	{
		ballYSpeed *= -1;
		racketHit++;
		ballXSpeed -= (shiftRacket - ballX)/50.0f;
		if(racketHit%10==0)
		{
			streamlineInfluence /= 10; 
			printf("next level\n");
		}
	}
	float diff;	
	if (winRatio > xyRatio)
		diff = (float)dataset->getChannel(0)->geom->getDimX()/(float)(g_iWidth-controlsWidth);
	else
		diff = (float)dataset->getChannel(0)->geom->getDimY()/(float)(g_iHeight-100);


	if(ballX <= (-1.0f*diff)+.05f)
	{
		ballXSpeed *= -1;
		ballX = (-1.0f*diff)+.05f;
	}
	else if(ballX >= (1.0f*diff)-.05f)
	{
		ballXSpeed *= -1;
		ballX = (1.0f*diff)-.05f;
	}
	
	
	if(ballY >= 0.95f)
	{
		if(ballX > .0f-(gateGap/2.0f) && ballX < .0f+(gateGap/2.0f))
		{
			if(gateGap < ballSize)
				ballYSpeed *= -1;
			else
			{
				printf("u suck!");				
				playGame = false;
				lost = true;
			}
		}
		else
			ballYSpeed *= -1;
	}
	else if(ballY < -.75)
	{
		printf("u suck!");				
		playGame = false;
		lost = true;
	}

	ballX -= dataset->getChannel(0)->getValueNormPos((ballX/2.0f)+.5f,(ballY/2.0f)+.5f)/streamlineInfluence;
	glClearColor(.0f,.0f,.0f,.0f);
	drawBall(ballX,ballY,ballSize);
	drawRacket(racketX,racketY,0.4f);
	drawGates(gateGap+=0.0001f);
	if(gateGap >= (1.0f*diff*2)*0.5f)
	{
		won = true;
		playGame = false;
	}
	drawBorders();
}

/**
* Draw ball for game.
* A point is drawn at the balls current location and a texture is put on it
* using point sprites.
*
* @param float x
* @param float y
* @param float size
*/
void drawBall(float x, float y, float size)
{
	glUniform1iARB(doShade, 0);
	glUniform1iARB(plotArrows, 0);
	glUniform1iARB(plotBall, 1);
	glUniform1iARB(arrowSize, 40);

	glEnable(GL_POINT_SPRITE);
	glEnable(GL_VERTEX_PROGRAM_POINT_SIZE);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glTexEnvf(GL_POINT_SPRITE_ARB,GL_COORD_REPLACE_ARB, GL_TRUE);

	glBegin(GL_POINTS);
	glVertex3f(x,y,-1.0f);
	glEnd();
	
	glDisable(GL_VERTEX_PROGRAM_POINT_SIZE);
	glDisable(GL_POINT_SPRITE);
	
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);

	glUniform1iARB(plotBall, 0);
	glUniform1iARB(arrowSize, arrow_size);
}

/**
* Draw Racket for Game.
* The racket is a red block at the bottom of the playing field, which
* can be moved with the arrow keys. The racket is used to make the ball
* bounce back and prevent it from falling down.
*
* @param float x
* @param float y
* @param float size
*/
void drawRacket(float x, float y, float size)
{
	x += shiftRacket;
	glColor3f(1.0f,0.0f,0.0f);
	glBegin(GL_QUADS);
	glVertex3f(x-(size/2.0f),y+0.05f,-1.0f);
	glVertex3f(x+(size/2.0f),y+0.05f,-1.0f);
	glVertex3f(x+(size/2.0f),y,-1.0f);
	glVertex3f(x-(size/2.0f),y,-1.0f);
	glEnd();
}

/**
* Draw Gates for Game.
* The gates form the top border of the image. At first this border is closed,
* but slowly the gates open, creating a gap through which the ball can go through.
*
* @param float gap
*/
void drawGates(float gap)
{
	float diff;	
	if (winRatio > xyRatio)
		diff = (float)dataset->getChannel(0)->geom->getDimX()/(float)(g_iWidth-controlsWidth);
	else
		diff = (float)dataset->getChannel(0)->geom->getDimY()/(float)(g_iHeight-100);

	glColor3f(1.0f,0.0f,0.0f);
	glBegin(GL_QUADS);
	glVertex3f(-1.0f*diff,1.0f,-1.0f);
	glVertex3f(0.0f-(gap/2.0f),1.0f,-1.0f);
	glVertex3f(0.0f-(gap/2.0f),0.95f,-1.0f);
	glVertex3f(-1.0f*diff,0.95f,-1.0f);
	glEnd();

	glColor3f(1.0f,0.0f,0.0f);
	glBegin(GL_QUADS);
	glVertex3f(0.0f+(gap/2.0f),1.0f,-1.0f);
	glVertex3f(1.0f*diff,1.0f,-1.0f);
	glVertex3f(1.0f*diff,0.95f,-1.0f);
	glVertex3f(0.0f+(gap/2.0f),0.95f,-1.0f);
	glEnd();
}

/**
* Draw Borders for Game.
* Borders are drawn around the image, which th ball cannot cross.
*
*/
void drawBorders()
{
	float diff;	
	if (winRatio > xyRatio)
		diff = (float)dataset->getChannel(0)->geom->getDimX()/(float)(g_iWidth-controlsWidth);
	else
		diff = (float)dataset->getChannel(0)->geom->getDimY()/(float)(g_iHeight-100);

	glColor3f(1.0f,0.0f,0.0f);
	glBegin(GL_QUADS);
	glVertex3f((-1.0f*diff)-.05f,1.0f,-1.0f);
	glVertex3f(-1.0f*diff,1.0f,-1.0f);
	glVertex3f(-1.0f*diff,-.75f,-1.0f);
	glVertex3f((-1.0f*diff)-.05f,-.75f,-1.0f);
	glEnd();

	glColor3f(1.0f,0.0f,0.0f);
	glBegin(GL_QUADS);
	glVertex3f(1.0f*diff,1.0f,-1.0f);
	glVertex3f((1.0f*diff)+.05f,1.0f,-1.0f);
	glVertex3f((1.0f*diff)+.05f,-.75f,-1.0f);
	glVertex3f(1.0f*diff,-.75f,-1.0f);
	glEnd();
}
//-----------------------------------THE GAME-----------------------------------//
//------------------------------------------------------------------------------//

//-------------------------------------------------------------------------------------------//
//---------------------------------------SHADERS---------------------------------------------//
/**
* Reads a textfile.
*
* @param char* fn
* @return char text
*/
char *textFileRead(char *fn) {

        FILE *fp;
        char *content = NULL;

        int f =0; int count;

        f = _open(fn, O_RDONLY);

        count = _lseek(f, 0, SEEK_END);

        _close(f);

        if (fn != NULL) {
                fp = fopen(fn,"rt");

                if (fp != NULL) {


                        if (count > 0) {
                                content = (char *)malloc(sizeof(char) * (count+1));
                                count = (int)fread(content,sizeof(char),count,fp);
                                content[count] = '\0';
                        }
                        fclose(fp);
                }
        }
        return content;
};

/**
* Set shaders for rendering textures.
* Here all shaders are initialized and settings made to
* ensure a functioning rendering of textures.
*
*/
void setShaders() {
	char *vShader,*fShader;

	vertex = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
	fragment = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);	

	vShader = textFileRead("Shader.vert");
	fShader = textFileRead("Shader.frag");

	const char * vvShader = vShader;
	const char * ffShader = fShader;

	glShaderSourceARB(vertex, 1, &vvShader,NULL);
	glShaderSourceARB(fragment, 1, &ffShader,NULL);

	free(vShader);
	free(fShader);

	glCompileShaderARB(vertex);
	glCompileShaderARB(fragment);

	GLchar * error = new GLchar[2001];
    GLsizei len;
    glGetShaderInfoLog(fragment, 2000, &len, error);
    std::cout<<"."<<error<<std::endl;

	p = glCreateProgramObjectARB();

	glAttachObjectARB(p,vertex);
	glAttachObjectARB(p,fragment);

	glLinkProgramARB(p);
	glUseProgramObjectARB(p);

};
//---------------------------------------SHADERS---------------------------------------------//
//-------------------------------------------------------------------------------------------//

//-------------------------------------------------------------------------------------------//
//---------------------------------GLUI BUTTONS----------------------------------------------//
/**
* Load new flow data.
* Opens a dialog to load a different data file and display its flow data.
*
*@param int control
*/
void openFile(int control) {

	//CLEAR ALL OLD DATA//
	//Clear all transfer funcion data - points, colors
	colorf = tf->clearData();
	delete tf;
	
	glBindTexture(GL_TEXTURE_1D, NULL);
	glBindTexture(GL_TEXTURE_2D, NULL);

	glDetachObjectARB(p,vertex); 
	glDetachObjectARB(p,fragment);
	glDeleteObjectARB(vertex);
	glDeleteObjectARB(fragment);
	glDeleteObjectARB(p); 
	//------------------------//


	OPENFILENAME ofn;       // common dialog box structure
	char szFile[260];       // buffer for file name
	szFile[0] = '\0';

	char szFileTitle[128];
	szFileTitle[0] = '\0';

	// Initialize OPENFILENAME
	ZeroMemory(&ofn, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = 0;
	ofn.lpstrFile = szFile;
	ofn.lpstrFile[0] = '\0';
	ofn.nMaxFile = sizeof(szFile);
	ofn.lpstrFilter = "Data\0*.GRI\0All\0*.*\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrFileTitle = szFileTitle;
	ofn.nMaxFileTitle = sizeof(szFileTitle);
	ofn.lpstrInitialDir = NULL;
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	// Display the Open dialog box.
	if (GetOpenFileName(&ofn)==TRUE)
	{
		GLenum glError = GL_NO_ERROR;

		std::string fileName = ofn.lpstrFile;

		//Load Volume and create new textures
		initTextures(fileName);

		//changed = true;
	}
}


/**
* Open color for transfer function.
* Opens a dialog to load saved color for the transfer
* function from a .pts file.
*
*@param int control
*/
void openColor(int control){
	OPENFILENAME ofn;       // common dialog box structure
	char szFile[260];       // buffer for file name
	szFile[0] = '\0';

	char szFileTitle[128];
	szFileTitle[0] = '\0';

	// Initialize OPENFILENAME
	ZeroMemory(&ofn, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = 0;
	ofn.lpstrFile = szFile;
	ofn.lpstrFile[0] = '\0';
	ofn.nMaxFile = sizeof(szFile);
	ofn.lpstrFilter = "Points\0*.pts\0All\0*.*\0";
	ofn.nFilterIndex = 1;
	ofn.lpstrFileTitle = szFileTitle;
	ofn.nMaxFileTitle = sizeof(szFileTitle);
	ofn.lpstrInitialDir = NULL;
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	// Display the Open dialog box.
	if (GetOpenFileName(&ofn)==TRUE)
	{
		GLenum glError = GL_NO_ERROR;
		std::string fileName = ofn.lpstrFile;

		ifstream file;
		file.open(fileName.c_str());
		if (!file){
			cout << "Error in openening file";
			//return EXIT_FAILURE;
		}
		tf->removePoints();
		tf->sortPoints();

		int x; float r, g, b; 
		string lineread;
		while(std::getline(file, lineread)) // Read line by line
		{
			cout << lineread << "\n";
			for(int i =0; i < 4; i++)
			{
				int index = (int)lineread.find_first_of(',');
				string s = string( lineread, 0, index);
				char *cstr = new char [s.size()+1];
				strcpy (cstr, s.c_str());
				float f = (float)atof(cstr);
				lineread = string( lineread, index+1, lineread.length());
				if(i==0)
					x=(int)f;
				else if(i==1)
					r=f;
				else if(i==2)
					g=f;
				else if(i==3)
					b=f;
				//printf("%d. = %f\n",i, f);
			}
			if(x != 0.0 && x != num)
			{
				Color *c = new Color(r,g,b);
				ColorPoint* dp = new ColorPoint(x, c);
				tf->addPoint(dp);
			}
		}
	}
}


/**
* Save collor from transfer function.
* Opens a dialog to save the created color from the transfer
* function to a .pts file.
*
*@param int control
*/
void saveColor(int control)
{
	OPENFILENAME ofn;
	char szFileName[MAX_PATH] = "";

	ZeroMemory(&ofn, sizeof(ofn));

	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = 0;
	ofn.lpstrFilter = "Points\0*.pts\0All\0*.*\0";
	ofn.lpstrFile = szFileName;
	ofn.nMaxFile = MAX_PATH;
	ofn.Flags = OFN_EXPLORER | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;

	if(GetSaveFileName(&ofn))
	{
		printf("saved!!");
		std::string fileName = ofn.lpstrFile;
		fileName.append(".pts");
		//cout << fileName;
		char *cstr = new char [fileName.size()+1];
		strcpy(cstr, fileName.c_str());
		ofstream myfile (cstr);
		if (myfile.is_open())
		{
			for(unsigned int i = 0; i < tf->points.size(); i++)
			{
				string str;
				char floatStringBuffer[256];
				sprintf(floatStringBuffer, "%f,", tf->points.at(i)->getX());
				str.append(floatStringBuffer);
				sprintf(floatStringBuffer, "%f,", tf->points.at(i)->getColor()->GetNormalizedRed());
				str.append(floatStringBuffer);
				sprintf(floatStringBuffer, "%f,", tf->points.at(i)->getColor()->GetNormalizedGreen());
				str.append(floatStringBuffer);
				sprintf(floatStringBuffer, "%f\n", tf->points.at(i)->getColor()->GetNormalizedBlue());
				str.append(floatStringBuffer);
				myfile << str;
			}
			myfile.close();
		}

		//WriteToFile(hwnd);
	}
}

/**
* Start Game.
* Callback function for the GLUI button which starts the game.
*
* @param int control
*/
void startGame(int control)
{
	
	if(lost==true)
	{
		lost=false;	
		won = false;
		shiftRacket=0;
		ballX=.0f, ballY=.0f, ballXSpeed=.0f, ballYSpeed=-0.02f;
		racketX=.0f, racketY=-.75f;
		gateGap=.0f;
		ballSize = .05f;
		racketHit = 0;
		streamlineInfluence = 10000.0f;
		playGame=true;
	}
	else
		playGame=true;
}

/**
* Callback function for GUI.
* This function takes the values from the radiobox selection. The data selected is displayed
* and color coded. Also the slider is updated, which lets you set the size of the arrows.
*
* @param int control
*/
void colorCoding(int control) {
	glUniform1iARB(useData, group1->get_int_val());
	slider->update_scroll_parameters();
	arrow_size = slider->get_int_val();
}

/**
* Draw streamlines from data.
* This function draws the streamlines which were calculated from a given dataset
* and saved in the vector 'lines'.
*
* @param std::vector<float*> lines
*/
void drawStreamlines(std::vector<float*> lines)
{
	float* line; float PoneX, PoneY, PtwoX, PtwoY;
	float diffX = (float)dataset->getChannel(0)->geom->getDimX()/(float)(g_iWidth-controlsWidth);

	for (int i = 0; i < lines.size(); i++)
	{
		line = lines[i];
		for (int j = 0; j < maxStreamLineSteps-3; j=j+2)
		{
 			PoneX = line[j]*2-1;
			PoneY = line[j+1]*2-1;
			PtwoX = line[j+2]*2-1; 
			PtwoY = line[j+3]*2-1;

			if (line[j] != -1 && line[j+1] != -1 && line[j+2] != -1 && line[j+3] != -1)
			{
				glBegin(GL_LINES);
				glVertex3f(PoneX*diffX,-PoneY,1.0f);
				glVertex3f(PtwoX*diffX,-PtwoY,1.0f);
				glEnd();
			}
			else
				break;
		}
	}
}

//---------------------------------GLUI BUTTONS----------------------------------------------//
//-------------------------------------------------------------------------------------------//

/**
* Initialize all textures and buffers.
* Creates all textures and buffers needed and perform all actions necessary 
* to display newly loaded data.
*
*@return int result
*/
int initTextures(std::string fname)
{
	//Remove old data
	delete dataset;
	//---------------------------------//
	//------------FROM MAIN------------//
	dataset = new FlowData();

	//load a dataset, an example path is given
	//bool result = dataset->loadDataset(fname,false);
	bool result = dataset->loadDataset("D:\\Uni\\Semester 7\\Visualisierung\\LU\\Bsp_2\\FlowVisFramework\\data\\hurricane_p_tc_singletime10",false);
	
	if(dataset->geometry.getIsFlipped())
		hurricane = 1;

	//exit if the loading was not successful
	if (!result) {system("PAUSE"); result = 1;}
	
	//copies the geometry into a new channel
	chX = dataset->createChannelGeometry(0);
	chY = dataset->createChannelGeometry(1);
	//in the data sets for the VisLU, the velocity vector is stored in the first three floats, thus channels 0, 1, and 2 contain its x, y, z components
	//this command creates a channel with the velocity magnitude
	vel = dataset->createChannelVectorLength(0,1,2);
	pre = 3; vor = 4;

	//aproximate velocity magnitude value in the middle
	//we take the velocity magnitude channel created at the previous line and ask for its value at position 0.5, 0.5 of normalized dataset coordinates (both dimensions scaled to floats from 0..1)
	//std::cout << dataset->getChannel(vel)->getValueNormPos(vec3(0.5,0.5)) << std::endl;
	//--------------FROM MAIN----------//
	//---------------------------------//

	//------------------------------------//
	//--------------TEXTURES--------------//
	GLenum glError = GL_NO_ERROR;

	//create texture
	glError = glGetError();

	//1D TExTURE
	//bind texture
	glActiveTexture(GL_TEXTURE1);
	glEnable(GL_TEXTURE_1D);
	glGenTextures(1, &g_newTexture);
	glBindTexture(GL_TEXTURE_1D, g_newTexture);
	
	glError = glGetError();
	if (glError != GL_NO_ERROR)
	{
		std::cerr<<"Error generating texture 1D: "<<gluErrorString(glError)<<std::endl;
	} 
	glDisable(GL_TEXTURE_1D);

	
	//2D TEXTURE
	float maxVel = dataset->getChannel(vel)->getMax();
	float maxPre = dataset->getChannel(pre)->getMax();
	float maxVor = dataset->getChannel(vor)->getMax();

	float minVel = dataset->getChannel(vel)->getMin();
	float minPre = dataset->getChannel(pre)->getMin();
	float minVor = dataset->getChannel(vor)->getMin();

	float rangeVel = dataset->getChannel(vel)->getRange();
	float rangePre = dataset->getChannel(pre)->getRange();
	float rangeVor = dataset->getChannel(vor)->getRange();

	float max, min, range;
	float velRange, velMin; 
	int pos;

	float* tex = new float[sizeX*sizeY*3];

	for (int ch = 0; ch < 3; ch++)
	{
		if (ch == 0)
		{
			max = maxVel; 
			min = minVel; range = rangeVel; pos = vel; 
			velRange=range; velMin=min;
		}
		else if (ch == 1)
		{
			max = maxPre; 
			min = minPre; range = rangePre; pos = pre;
		}
		else if (ch == 2)
		{
			max = maxVor; 
			min = minVor; range = rangeVor; pos = vor;
		}
			
		for(int i = 0; i<sizeY; i++) {
			for(int j = 0; j<sizeX*3; j+=3) {
				float b = ((dataset->getChannel(pos)->getValueNormPos(((float)j)/(sizeX*3.0f),((float)i)/sizeY))-min)/range;
				tex[(sizeY-i-1)*sizeX*3+j+ch] = b;				
			}
		}
	}

	//bind texture
	glActiveTexture(GL_TEXTURE0);
	glEnable(GL_TEXTURE_2D);
	glGenTextures(1, &g_uExampleTexture);
	glBindTexture(GL_TEXTURE_2D, g_uExampleTexture);
	
	glError = glGetError();
	if (glError != GL_NO_ERROR)
	{
		std::cerr<<"Error generating texture 2D: "<<gluErrorString(glError)<<std::endl;
	} 
	else if (g_uExampleTexture)
	{
		glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,sizeX,sizeY,0,GL_RGB,GL_FLOAT,(void *)tex);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);

		glError = glGetError();
		if (glError != GL_NO_ERROR)
		{
			std::cerr<<"Could not upload texture: "<<gluErrorString(glError)<<std::endl;
		}
	}
	glDisable(GL_TEXTURE_2D);


	//ARROW PLOT TEXTURE//
	ILuint arrowImage;
	ilGenImages(1, &arrowImage);
	ilBindImage(arrowImage);

	ilLoadImage("arrow.png");
	ILuint Width, Height;
	Width = ilGetInteger(IL_IMAGE_WIDTH);
	Height = ilGetInteger(IL_IMAGE_HEIGHT); 
	BYTE* Data = new BYTE[Width*Height*4];
	ilCopyPixels(0,0,0,Width,Height,1,IL_RGBA,IL_UNSIGNED_BYTE, Data);

    if( arrowImage != NULL )

	{
		glActiveTexture(GL_TEXTURE2);
		glEnable(GL_TEXTURE_2D);

		glGenTextures( 1, &g_ArrowTexture );
		glBindTexture( GL_TEXTURE_2D, g_ArrowTexture );

		glTexParameteri( GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR );
		glTexParameteri( GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);

		glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, Width, Height, 0,
				     GL_RGBA, GL_UNSIGNED_BYTE, Data );
	}

	//BALL TEXTURE//
	ILuint ballImage;
	ilGenImages(1, &ballImage);
	ilBindImage(ballImage);

	ilLoadImage("ball.png");

	Width = ilGetInteger(IL_IMAGE_WIDTH);
	Height = ilGetInteger(IL_IMAGE_HEIGHT); 
	Data = new BYTE[Width*Height*4];
	ilCopyPixels(0,0,0,Width,Height,1,IL_RGBA,IL_UNSIGNED_BYTE, Data);

	if( ballImage != NULL )

	{
		glActiveTexture(GL_TEXTURE4);
		glEnable(GL_TEXTURE_2D);

		glGenTextures( 1, &g_BallTexture );
		glBindTexture( GL_TEXTURE_2D, g_BallTexture );

		glTexParameteri( GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR );
		glTexParameteri( GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);

		glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, Width, Height, 0,
				     GL_RGBA, GL_UNSIGNED_BYTE, Data );
	}


	//Velocity Texture
	float* xyz = new float[sizeX*sizeY*3];

	//create texture
	glError = glGetError();

	//2D TExTURE
	//bind texture
	glActiveTexture(GL_TEXTURE3);
	glEnable(GL_TEXTURE_2D);
	glGenTextures(1, &velTexture);
	glBindTexture(GL_TEXTURE_2D, velTexture);
	
	glError = glGetError();
	if (glError != GL_NO_ERROR)
	{
		std::cerr<<"Error generating texture 3D: "<<gluErrorString(glError)<<std::endl;
	} 

	for(int i = 0; i<sizeY; i++) {
		for(int j = 0; j<sizeX*3; j+=3) {
			if(tex[(sizeY-i-1)*sizeX*3+j]!=0)
			{
				float a = (float)dataset->getChannel(0)->getValueNormPos(((float)j)/(sizeX*3),((float)i)/sizeY)/((tex[(sizeY-i-1)*sizeX*3+j]*velRange)+velMin);
				float b = (float)dataset->getChannel(1)->getValueNormPos(((float)j)/(sizeX*3),((float)i)/sizeY)/((tex[(sizeY-i-1)*sizeX*3+j]*velRange)+velMin);

				xyz[(sizeY-i-1)*sizeX*3+j] = a;
				xyz[(sizeY-i-1)*sizeX*3+j+1] = b;

				xyz[(sizeY-i-1)*sizeX*3+j] *= 0.5f;
				xyz[(sizeY-i-1)*sizeX*3+j+1] *= 0.5f;

				xyz[(sizeY-i-1)*sizeX*3+j] += 0.5f;
				xyz[(sizeY-i-1)*sizeX*3+j+1] += 0.5f;				
			}
			else
			{
				xyz[(sizeY-i-1)*sizeX*3+j] = 0.0f;
				xyz[(sizeY-i-1)*sizeX*3+j+1] = 0.0f;
			}
		}
	}

	glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,sizeX,sizeY,0,GL_RGB,GL_FLOAT,(void *)xyz);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	glDisable(GL_TEXTURE_2D);
	//--------------TEXTURES--------------//
	//------------------------------------//
	

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

	num = dataset->getChannel(vel)->geom->getDimX() * dataset->getChannel(vel)->geom->getDimY() / dividedBy;
	tf = new Transfer(num);
	//Get 1D texture
	colorf = tf->colorInterpolation();

	//SHADER Data//
	GLint flow_sampler = glGetUniformLocationARB(p, "flow_sampler");
	glUniform1iARB(flow_sampler, 0);

	GLint transfer_sampler = glGetUniformLocationARB(p, "transfer_sampler");
	glUniform1iARB(transfer_sampler, 1);

	GLint arrow_sampler = glGetUniformLocationARB(p, "arrow_sampler");
	glUniform1iARB(arrow_sampler, 2);

	GLint velValue = glGetUniformLocationARB(p, "xyz");
	glUniform1iARB(velValue, 3);

	GLint ball_sampler = glGetUniformLocationARB(p, "ball_sampler");
	glUniform1iARB(ball_sampler, 4);

	doShade = glGetUniformLocationARB(p, "shade");
	glUniform1iARB(doShade, 0);

	useData = glGetUniformLocationARB(p, "data");
	glUniform1iARB(doShade, 0);

	plotArrows = glGetUniformLocationARB(p, "arrow");
	glUniform1iARB(plotArrows, 0);

	GLint size = glGetUniformLocationARB(p, "size");
	glUniform1iARB(size, 15.0);

	arrowAngle = glGetUniformLocationARB(p, "angle");
	glUniform1iARB(arrowAngle, 90);

	drawHurricane = glGetUniformLocationARB(p, "drawHurricane");
	glUniform1iARB(drawHurricane, hurricane);

	adaptPtSize = glGetUniformLocationARB(p, "adaptSize");
	glUniform1iARB(adaptPtSize, adaptPt);

	arrowSize = glGetUniformLocationARB(p, "size");
	glUniform1iARB(arrowSize, arrow_size);

	plotBall = glGetUniformLocationARB(p, "ball");
	glUniform1iARB(plotBall, 0);

	return result;
};

int main(int argc, char** argv)
{
	printf("Hello student :)\n");
	
	//---------------------------//
	//I AM TOO OLD FOR THIS STUFF//
	std::cout << "- Initializing GLUT ..." << std::endl;
	glutInit (&argc, argv);

	glutSetOption(GLUT_ACTION_ON_WINDOW_CLOSE,GLUT_ACTION_GLUTMAINLOOP_RETURNS);
	glutInitWindowSize (g_iWidth,g_iHeight);
	glutInitDisplayMode (GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH | GLUT_ALPHA);

	main_window = glutCreateWindow("VisLuFramework");
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glViewport(0, 0, g_iWidth, g_iHeight);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	const unsigned char vpMessage[] = {"Initializing (see console for details) ..."};
	const int iMessageWidth = glutBitmapLength(GLUT_BITMAP_HELVETICA_12,vpMessage);
	const int iMessageHeight = glutBitmapHeight(GLUT_BITMAP_HELVETICA_12);

	glColor4f(1.0f,1.0f,1.0f,1.0f);
	glRasterPos2f(-(float(iMessageWidth)/float(g_iWidth)),(float(iMessageHeight)/float(g_iHeight)));
	glutBitmapString(GLUT_BITMAP_HELVETICA_12,vpMessage);
	glRasterPos2f(0.0f,0.0f);

	//set glut (callback-) functions
	GLUI_Master.set_glutIdleFunc (glutIdle);
	GLUI_Master.set_glutDisplayFunc (glutDisplay);
	GLUI_Master.set_glutReshapeFunc (glutReshape);
	GLUI_Master.set_glutMouseFunc (glutMouse);
	glutMotionFunc (glutMotion);
	GLUI_Master.set_glutKeyboardFunc (glutKeyboard);
	glutSpecialFunc(glutSpecialKey);

	std::cout << "- GLUT initialized." << std::endl << std::endl;

	// Initialize GLEW
	std::cout << "- Initializing GLEW ..." << std::endl;
	GLenum err = glewInit();
	if (err != GLEW_OK)
	{
		// glewInit failed, something is seriously wrong
		std::cerr << "Error initializing GLEW: " << glewGetErrorString(err) << std::endl;
		return EXIT_FAILURE;
	}
	std::cout << "- GLEW initialized." << std::endl << std::endl;

	// Initialize DevIL (an image library)
	std::cout << "- Initializing DevIL ..." << std::endl;
	ilInit();
	if (ilGetError() != IL_NO_ERROR)
	{
		std::cerr << "Error initializing DevIL" << std::endl;
		return EXIT_FAILURE;
	}
	ilEnable(IL_ORIGIN_SET);
	ilOriginFunc(IL_ORIGIN_LOWER_LEFT);
	std::cout << "- DevIL initialized." << std::endl << std::endl;
	//I AM TOO OLD FOR THIS STUFF//
	//---------------------------//

	//GLUI 
	GLUI_Master.set_glutReshapeFunc( glutReshape );
    glui = GLUI_Master.create_glui_subwindow(main_window, GLUI_SUBWINDOW_RIGHT);

	new GLUI_StaticText(glui, "");
    new GLUI_Button( glui, "Open", 0, openFile );
	new GLUI_StaticText(glui, "");
    new GLUI_Button( glui, "Close", 0, glutClose );

	new GLUI_StaticText(glui, "");
	new GLUI_StaticText(glui, "");

	GLUI_Panel* panel = new GLUI_Panel(glui, "Display data", 1);

	new GLUI_StaticText(panel, "");
	new GLUI_Checkbox(panel,"Arrow Plot",&drawArrows);
	slider = new GLUI_Scrollbar(panel, "Arrow Size", GLUI_SCROLL_HORIZONTAL, GLUI_SCROLL_INT, 1, colorCoding);
	slider->set_int_limits(0, 30, GLUI_LIMIT_CLAMP);
	slider->set_int_val(arrow_size);
	GLUI_Checkbox* cb = new GLUI_Checkbox(panel,"Adapt Size",&adaptPt);
	cb->set_alignment(GLUI_ALIGN_RIGHT);

    new GLUI_StaticText(panel, "");
	new GLUI_Checkbox(panel,"Euler-Streamlines",&drawEuler);
    new GLUI_StaticText(panel, "");
	new GLUI_Checkbox(panel,"RK-Streamlines",&drawRK);
    new GLUI_StaticText(panel, "");
    new GLUI_StaticText(panel, "");
    new GLUI_Separator(panel);
    new GLUI_StaticText(panel, "");

    new GLUI_StaticText(panel, "Color coding:");
    new GLUI_StaticText(panel, "");

	group1 = new GLUI_RadioGroup(panel,0,1,colorCoding);
    new GLUI_RadioButton( group1, "Velocity" );
    new GLUI_RadioButton( group1, "Pressure" );
    new GLUI_RadioButton( group1, "Vorticity" );

    new GLUI_StaticText(panel, "");

    new GLUI_Button(panel, "Open Color", 0, openColor );
    new GLUI_Button(panel, "Save Color", 0, saveColor );

    new GLUI_StaticText(panel, "");

	new GLUI_Button(glui, "Start Game", 0, startGame );

	glui->set_main_gfx_window( main_window);
	//GLUI//

	//TEXTURES//
	//1D//
	glActiveTexture(GL_TEXTURE1);
	glEnable(GL_TEXTURE_1D);
	glGenTextures(1, &g_newTexture);

	//2D//
	glActiveTexture(GL_TEXTURE0);
	glEnable(GL_TEXTURE_2D);
	glGenTextures(1, &g_uExampleTexture);

	//SHADERS//
	setShaders();

	std::string fname = "c_block";
	int result = initTextures(fname);

	// Enter the main loop
	glutMainLoop();

	//system("PAUSE");
	std::cout << "- Main loop terminated." << std::endl << std::endl;
	//return EXIT_SUCCESS;
	return result;
}